@@ -1,17 +1,20 @@
-#' @title Double sigmoidal fit function.
+#' @title Double sigmoidal fit function with h0.
 #'
-#' @param dataInput A data frame or a list contatining the dataframe. The data frame should be composed of at least two columns. One represents time, and the other represents intensity. The data should be normalized with the normalize data function sicegar::normalizeData() before imported into this function.
+#' @param dataInput A dataframe or a list containing the dataframe. The dataframe should be composed of at least two columns. One represents time, and the other represents intensity. The data should be normalized with the normalize data function sicegar::normalizeData() before being imported into this function.
 #' @param tryCounter A counter that shows the number of times the data was fit via maximum likelihood function.
-#' @param startList The initial set of parameters vector that algorithm tries for the first fit attempt for the relevant parameters. The vector composes of six elements; 'finalAsymptoteIntensityRatio', 'maximum', 'slope1Param', 'midPoint1Param' , 'slope2Param', and 'midPointDistanceParam'. Detailed explanations of those parameters can be found in vignettes. Defaults are  finalAsymptoteIntensityRatio = 0, maximum = 1, slope1Param = 1, midPoint1Param = 0.33, slope2Param = 1, and midPointDistanceParam=0.29. The numbers are in normalized time intensity scale.
-#' @param lowerBounds The lower bounds for the randomly generated start parameters.  The vector composes of six elements; 'finalAsymptoteIntensityRatio', 'maximum', 'slope1Param', 'midPoint1Param' , 'slope2Param', and 'midPointDistanceParam'. Detailed explanations of those parameters can be found in vignettes. Defaults are finalAsymptoteIntensityRatio = 0, maximum = 0.3, slope1Param = .01, midPoint1Param = -0.52, slope2Param = .01, and midPointDistanceParam = 0.04. The numbers are in normalized time intensity scale.
-#' @param upperBounds The upper bounds for the randomly generated start parameters.  The vector composes of six elements; 'finalAsymptoteIntensityRatio', 'maximum', 'slope1Param', 'midPoint1Param' , 'slope2Param', and 'midPointDistanceParam'. Detailed explanations of those parameters can be found in vignettes. Defaults are finalAsymptoteIntensityRatio = 1, maximum = 1.5, slope1Param = 180, midPoint1Param = 1.15, slope2Param = 180, and midPointDistanceParam = 0.63. The numbers are in normalized time intensity scale.
+#' @param startList The vector containing the initial set of parameters that algorithm tries for the first fit attempt for the relevant parameters. The vector composes of seven elements; 'finalAsymptoteIntensityRatio', 'maximum', 'slope1Param', 'midPoint1Param' , 'slope2Param', 'midPointDistanceParam', and 'h0'. Detailed explanations of those parameters can be found in vignettes. Defaults are  finalAsymptoteIntensityRatio = 0, maximum = 1, slope1Param = 1, midPoint1Param = 0.33, slope2Param = 1, midPointDistanceParam=0.29, and h0 = 0. The numbers are in normalized time intensity scale.
+#' @param lowerBounds The lower bounds for the randomly generated start parameters. The vector is composed of seven elements; 'finalAsymptoteIntensityRatio', 'maximum', 'slope1Param', 'midPoint1Param' , 'slope2Param', 'midPointDistanceParam', and 'h0'. Detailed explanations of those parameters can be found in vignettes. Defaults are finalAsymptoteIntensityRatio = 0, maximum = 0.3, slope1Param = .01, midPoint1Param = -0.52, slope2Param = .01, midPointDistanceParam = 0.04, and h0 = -0.1. The numbers are in normalized time intensity scale.
+#' @param upperBounds The upper bounds for the randomly generated start parameters. The vector is composed of seven elements; 'finalAsymptoteIntensityRatio', 'maximum', 'slope1Param', 'midPoint1Param' , 'slope2Param', 'midPointDistanceParam', and 'h0'. Detailed explanations of those parameters can be found in vignettes. Defaults are finalAsymptoteIntensityRatio = 1, maximum = 1.5, slope1Param = 180, midPoint1Param = 1.15, slope2Param = 180, midPointDistanceParam = 0.63, and h0 = 0.3. The numbers are in normalized time intensity scale.
 #' @param min_Factor Defines the minimum step size used by the fitting algorithm. Default is 1/2^20.
-#' @param n_iterations Define maximum number of iterations used by the fitting algorithm. Default is 1000
+#' @param n_iterations Defines maximum number of iterations used by the fitting algorithm. Default is 1000.
 #'
-#' @description The function fits a double sigmoidal curve to given data by using likelihood maximization (LM) algorithm and provides the parameters (maximum, final asymptote intensity, slope1Param, midpoint1Param, slope2Param, and midpoint distance) describing the double-sigmoidal fit as output. It also contains information about the goodness of fits such as AIC, BIC, residual sum of squares, and log likelihood.
+#' @description The function fits a double sigmoidal curve to given data by using likelihood maximization (LM) algorithm and provides the parameters (maximum, final asymptote intensity, slope1Param, midpoint1Param, slope2Param, and midpoint distance) describing the double sigmoidal fit as output. It also contains information about the goodness of fits such as AIC, BIC, residual sum of squares, and log likelihood.
 #' @return Returns the fitted parameters and goodness of fit metrics.
 #' @export
 #' @examples
+#' # runif() is used here for consistency with previous versions of the sicegar package. However,
+#' # rnorm() will generate symmetric errors, producing less biased numerical parameter estimates.
+#' # We recommend errors generated with rnorm() for any simulation studies on sicegar.
 #'time=seq(3, 24, 0.1)
 #'
 #'#simulate intensity data and add noise
@@ -116,7 +119,7 @@ doublesigmoidalFitFunction <- function(dataInput,
                                         upper = upperBounds,
                                         trace=F), silent = TRUE)
 
-  if(class(theFitResult) != "try-error")
+  if(!inherits(theFitResult, "try-error"))
   {
     parameterMatrix <- summary(theFitResult)$parameters
     colnames(parameterMatrix) <- c("Estimate", "Std_Error", "t_value", "Pr_t")
@@ -150,7 +153,7 @@ doublesigmoidalFitFunction <- function(dataInput,
 
   }
 
-  if(class(theFitResult) == "try-error")
+  if(inherits(theFitResult, "try-error"))
   {
     parameterVector <- rep(NA, 24)
     names(parameterVector) <- c("finalAsymptoteIntensityRatio_N_Estimate", "finalAsymptoteIntensityRatio_Std_Error", "finalAsymptoteIntensityRatio_t_value", "finalAsymptoteIntensityRatio_Pr_t",
@@ -189,18 +192,21 @@ doublesigmoidalFitFunction <- function(dataInput,
 #**************************************
 #' @title Double Sigmoidal Formula
 #'
-#' @param x the "time" (time) column of the dataframe
-#' @param finalAsymptoteIntensityRatio This is the ratio between asymptote intensity and maximum intensity of the fitted curve.
-#' @param maximum the maximum intensity that the double sigmoidal function reach.
-#' @param slope1Param the slope parameter of the sigmoidal function at the steppest point in the exponential phase of the viral production.
-#' @param midPoint1Param the x axis value of the steppest point in the function.
-#' @param slope2Param the slope parameter of the sigmoidal function at the steppest point in the lysis phase. i.e when the intensity is decreasing.
-#' @param midPointDistanceParam the distance between the time of steppest increase and steppest decrease in the intensity data. In other words the distance between the x axis values of arguments of slope1Param and slope2Param.
+#' @param x the "time" column of the dataframe
+#' @param finalAsymptoteIntensityRatio this is the ratio between the final asymptote intensity and maximum intensity of the fitted curve.
+#' @param maximum the maximum intensity that the double sigmoidal function reaches.
+#' @param slope1Param the slope parameter of the sigmoidal function at the steepest point in the exponential phase of the viral production.
+#' @param midPoint1Param the x axis value of the steepest point in the function.
+#' @param slope2Param the slope parameter of the sigmoidal function at the steepest point in the lysis phase. i.e when the intensity is decreasing.
+#' @param midPointDistanceParam the distance between the time of steepest increase and steepest decrease in the intensity data. In other words the distance between the x axis values of arguments of slope1Param and slope2Param.
 #'
-#' @description Calculates intensities using the double-sigmoidal model fit and the parameters (maximum, final asymptote intensity, slope1Param, midpoint1Param, slope2Param, and mid point distance).
-#' @return Returns the predicted intensities for the given time points with the double-sigmoidal fitted parameters for the double sigmoidal fit.
+#' @description Calculates intensities using the double sigmoidal model fit and the parameters (maximum, final asymptote intensity, slope1Param, midpoint1Param, slope2Param, and midpoint distance).
+#' @return Returns the predicted intensities for the given time points with the double sigmoidal fitted parameters for the double sigmoidal fit.
 #' @export
 #' @examples
+#' # runif() is used here for consistency with previous versions of the sicegar package. However,
+#' # rnorm() will generate symmetric errors, producing less biased numerical parameter estimates.
+#' # We recommend errors generated with rnorm() for any simulation studies on sicegar.
 #'time <- seq(3, 24, 0.1)
 #'
 #'#simulate intensity data and add noise
@@ -261,7 +267,7 @@ doublesigmoidalFitFormula <- function(x,
     stop("slope2Param should be a positive number. It is the absolute value of the second slopeParam")
     }
   if(midPointDistanceParam < 0){
-    stop("midPointDistanceParam should be a positive number. It is the distance between two steppest points of exponential phase and lysis")
+    stop("midPointDistanceParam should be a positive number. It is the distance between two steepest points of exponential phase and lysis")
     }
   if(finalAsymptoteIntensityRatio < 0 | finalAsymptoteIntensityRatio > 1){
     stop("finalAsymptoteIntensityRatio should be a number between 0 and 1")
@@ -373,7 +379,7 @@ f_argmax_doublesigmoidal <- function(parameterVector){
     stop("slope2Param should be a positive number. It is the absolute value of the slope2Param")
     }
   if(midPointDistanceParam < 0){
-    stop("midPointDistanceParam should be a positive number. It is the distance between two steppest points of exponential phase and lysis")
+    stop("midPointDistanceParam should be a positive number. It is the distance between two steepest points of exponential phase and lysis")
     }
   if(finalAsymptoteIntensityRatio < 0 | finalAsymptoteIntensityRatio > 1){
     stop("finalAsymptoteIntensityRatio should be a number between 0 and 1")
@@ -459,11 +465,11 @@ f_mid1_doublesigmoidal <- function(parameterDf){
                                      M1 = parameterDf$midPoint1Param_Estimate,
                                      B2 = parameterDf$slope2Param_Estimate,
                                      L = parameterDf$midPointDistanceParam_Estimate, const = constt), silent = TRUE);
-  if(class(mid1x) == "try-error")
+  if(inherits(mid1x, "try-error"))
   {
     rangeList <- seq(-1,30)
     rangeListCounter <- -1
-    while(class(mid1x) == "try-error" & rangeListCounter <= 30)
+    while(inherits(mid1x, "try-error") && rangeListCounter <= 30)
     {
       #print(rangeListCounter)
       rangeListSelection <- rangeList[rangeListCounter + 2];
@@ -552,10 +558,10 @@ f_mid2_doublesigmoidal <- function(parameterDf){
                               B2 = parameterDf$slope2Param_Estimate,
                               L = parameterDf$midPointDistanceParam_Estimate, const=constt), silent = TRUE);
 
-  if(class(mid2x) == "try-error"){
+  if(inherits(mid2x, "try-error")){
     rangeList <- seq(-1,30)
     rangeListCounter <- -1
-    while(class(mid2x) == "try-error" & rangeListCounter <= 30){
+    while(inherits(mid2x, "try-error") && rangeListCounter <= 30){
       rangeListSelection <- rangeList[rangeListCounter + 2];
       mid2x <- try(stats::uniroot(f0mid,
                                   interval = c(argumentt, max_x * (2^rangeListSelection)),
@@ -593,7 +599,7 @@ f_mid2_doublesigmoidal <- function(parameterDf){
 # @param timeStep is the time step for the derivative calculation increment "h". Default value is 0.00001
 #
 #
-# @description The function calculates the numerical slope of double sigmoidal function with given parameters by using 5 points derivative. It is designed to compansate for the small discrepancy in numerical parameters in the double sigmoidal model. The function is called by numericalReCalculation.
+# @description The function calculates the numerical slope of double sigmoidal function with given parameters by using 5 points derivative. It is designed to compensate for the small discrepancy in numerical parameters in the double sigmoidal model. The function is called by numericalReCalculation.
 # @return return the numerical slope of double sigmoidal for the given x value.
 # @export
 #
@@ -688,24 +694,25 @@ f_slope_doublesigmoidal <- function(x, parameterDf, timeStep = 0.00001){
 
 #**************************************
 # @title sigmoidalRenormalizeParameters (This is an internal function)
-# @param parametersDf it is the parameter data frame generated by sigmoidal fit function
+# @param parametersDf is the parameter data frame generated by sigmoidal fit function and
 #        includes the parameters named
 #        *maximum_N_Estimate (normalized Maximum Estimate)
-#        *finalAsymptoteIntensityRatio_N_Estimate (Normalized final asymtote intensity estimate)
-#        *slope1Param_N_Estimate (normalzied Slope1Param Estimate)
+#        *finalAsymptoteIntensityRatio_N_Estimate (normalized final asymptote intensity estimate)
+#        *slope1Param_N_Estimate (normalized Slope1Param Estimate)
 #        *midPoint1Param_N_Estimate (normalized Midpoint1Param Estimate)
-#        *slope2Param_N_Estimate (normalzied Slope2Param Estimate)
+#        *slope2Param_N_Estimate (normalized Slope2Param Estimate)
 #        *midPointDistanceParam_N_Estimate (normalized distance between midpoints estimate)
 #        *dataScalingParameters.intensityRange the range of initial unnormalized intensity. Provided if the data is normalized
 #        *parameterDF$dataScalingParameters.intensityMin the minimum of unnormalized intensity. Provided if the data is normalized
-#        *parameterDF$dataScalingParameters.timeRange tha maximum time that the experiment reach. Provided if the data is normalized
-# @param isalist defines if the input is provided as a list (i.e normalized) or as a data frame (i.e not normalized)
-# @details If the fit was done in normalized data frame then the found fit parameters will belong to normalized data.
-#          This function generates unnormalized counterparts of those parameters.
+#        *parameterDF$dataScalingParameters.timeRange the maximum time that the experiment reach. Provided if the data is normalized
+# @param isalist checks if the input provided is a list (i.e normalized) or a data frame (i.e not normalized)
+# @details If the fit was done on normalized data, parameter estimates will be on normalized scale.
+#          This function unnormalizes those parameter estimations.
 doublesigmoidalRenormalizeParameters <- function(parameterDF, isalist)
 {
   if(isalist){
-    parameterDF$finalAsymptoteIntensityRatio_Estimate <- parameterDF$finalAsymptoteIntensityRatio_N_Estimate
+    parameterDF$finalAsymptoteIntensityRatio_Estimate <- (parameterDF$finalAsymptoteIntensityRatio_N_Estimate * parameterDF$maximum_N_Estimate * parameterDF$dataScalingParameters.intensityRange + parameterDF$dataScalingParameters.intensityMin) /
+                                                              (parameterDF$maximum_N_Estimate  * parameterDF$dataScalingParameters.intensityRange + parameterDF$dataScalingParameters.intensityMin)
     parameterDF$maximum_Estimate <- parameterDF$maximum_N_Estimate * parameterDF$dataScalingParameters.intensityRange + parameterDF$dataScalingParameters.intensityMin
     parameterDF$slope1Param_Estimate <- parameterDF$slope1Param_N_Estimate / parameterDF$dataScalingParameters.timeRange
     parameterDF$midPoint1Param_Estimate <- parameterDF$midPoint1Param_N_Estimate * parameterDF$dataScalingParameters.timeRange
